@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -23,6 +23,12 @@
 
 #include <fluent-bit/flb_info.h>
 
+#ifdef FLB_SYSTEM_WINDOWS
+#include <monkey/mk_core/external/winpthreads.h>
+#else
+#include <pthread.h>
+#endif
+
 /* Ideal case when the compiler support direct storage through __thread */
 #ifdef FLB_HAVE_C_TLS
 #define FLB_TLS_SET(key, val)      key=val
