@@ -65,7 +65,8 @@ static int unpack_and_pack(msgpack_packer *pck, msgpack_object *root,
     if (val != NULL) {
         msgpack_pack_str(pck, val_len);
         msgpack_pack_str_body(pck, val, val_len);
-    } else {
+    }
+    else {
         msgpack_pack_uint64(pck, val_uint64);
     }
 
@@ -231,12 +232,18 @@ static int process_content(struct flb_tail_file *file, size_t *bytes)
     /* Parse the data content */
     data = file->buf_data;
     end = data + file->buf_len;
-    while ((p = memchr(data, '\n', end - data))) {
-        len = (p - data);
 
+    /* Skip null characters from the head (sometimes introduced by copy-truncate log rotation) */
+    while (data < end && *data == '\0') {
+        data++;
+        processed_bytes++;
+    }
+
+    while (data < end && (p = memchr(data, '\n', end - data))) {
+        len = (p - data);
         if (file->skip_next == FLB_TRUE) {
             data += len + 1;
-            processed_bytes += len;
+            processed_bytes += len + 1;
             file->skip_next = FLB_FALSE;
             continue;
         }
@@ -352,14 +359,22 @@ static int process_content(struct flb_tail_file *file, size_t *bytes)
         lines++;
     }
     file->parsed = file->buf_len;
-    *bytes = processed_bytes;
 
-    /* Append buffer content to a chunk */
-    flb_input_chunk_append_raw(ctx->ins,
-                               file->tag_buf,
-                               file->tag_len,
-                               out_sbuf->data,
-                               out_sbuf->size);
+    if (lines > 0) {
+        /* Append buffer content to a chunk */
+        *bytes = processed_bytes;
+        flb_input_chunk_append_raw(ctx->ins,
+                                   file->tag_buf,
+                                   file->tag_len,
+                                   out_sbuf->data,
+                                   out_sbuf->size);
+    }
+    else if (file->skip_next) {
+        *bytes = file->buf_len;
+    }
+    else {
+        *bytes = processed_bytes;
+    }
 
     msgpack_sbuffer_destroy(out_sbuf);
     return lines;
@@ -937,6 +952,7 @@ int flb_tail_file_chunk(struct flb_tail_file *file)
             }
 
             /* Do buffer adjustments */
+            file->offset += file->buf_len;
             file->buf_len = 0;
             file->skip_next = FLB_TRUE;
         }
