@@ -2,7 +2,7 @@
 
 /*  Fluent Bit
  *  ==========
- *  Copyright (C) 2019-2020 The Fluent Bit Authors
+ *  Copyright (C) 2019-2021 The Fluent Bit Authors
  *  Copyright (C) 2015-2018 Treasure Data Inc.
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
@@ -746,6 +746,8 @@ static int forward_config_ha(const char *upstream_file,
         flb_upstream_node_set_data(fc, node);
     }
 
+    flb_output_upstream_ha_set(ctx->ha, ctx->ins);
+
     return 0;
 }
 
@@ -797,7 +799,7 @@ static int forward_config_simple(struct flb_forward *ctx,
     upstream = flb_upstream_create(config,
                                    ins->host.name,
                                    ins->host.port,
-                                   io_flags, (void *) &ins->tls);
+                                   io_flags, ins->tls);
     if (!upstream) {
         flb_free(fc);
         flb_free(ctx);
