@@ -40,6 +40,74 @@
 #include <mbedtls/base64.h>
 #include <mbedtls/sha256.h>
 
+pthread_key_t oauth2_type;
+pthread_key_t oauth2_token;
+
+int is_tag_match_regex(struct flb_stackdriver *ctx, const char *tag, int tag_len);
+int is_local_resource_id_match_regex(struct flb_stackdriver *ctx);
+int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
+                                       const char *tag, int tag_len,
+                                       int from_tag);
+
+static void oauth2_cache_exit(void *ptr)
+{
+    if (ptr) {
+        flb_sds_destroy(ptr);
+    }
+}
+
+static void oauth2_cache_init()
+{
+    /* oauth2 pthread key */
+    pthread_key_create(&oauth2_type, oauth2_cache_exit);
+    pthread_key_create(&oauth2_token, oauth2_cache_exit);
+}
+
+/* Set oauth2 type and token in pthread keys */
+static void oauth2_cache_set(char *type, char *token)
+{
+    flb_sds_t buf_type;
+    flb_sds_t buf_token;
+
+    /* oauth2 type */
+    buf_type = pthread_getspecific(oauth2_type);
+    if (buf_type) {
+        flb_sds_destroy(buf_type);
+    }
+    buf_type = flb_sds_create(type);
+    pthread_setspecific(oauth2_type, buf_type);
+
+    /* oauth2 access token */
+    buf_token = pthread_getspecific(oauth2_token);
+    if (buf_token) {
+        flb_sds_destroy(buf_token);
+    }
+    buf_token = flb_sds_create(token);
+    pthread_setspecific(oauth2_token, buf_token);
+}
+
+/* By using pthread keys cached values, compose the authorizatoin token */
+static flb_sds_t oauth2_cache_to_token()
+{
+    flb_sds_t type;
+    flb_sds_t token;
+    flb_sds_t output;
+
+    type = pthread_getspecific(oauth2_type);
+    if (!type) {
+        return NULL;
+    }
+
+    output = flb_sds_create(type);
+    if (!output) {
+        return NULL;
+    }
+
+    token = pthread_getspecific(oauth2_token);
+    flb_sds_printf(&output, " %s", token);
+    return output;
+}
+
 /*
  * Base64 Encoding in JWT must:
  *
@@ -264,7 +332,18 @@ static flb_sds_t get_google_token(struct flb_stackdriver *ctx)
     int ret = 0;
     flb_sds_t output = NULL;
 
-    if (pthread_mutex_lock(&ctx->token_mutex)){
+    ret = pthread_mutex_trylock(&ctx->token_mutex);
+    if (ret == EBUSY) {
+        /*
+         * If the routine is locked we just use our pre-cached values and
+         * compose the expected authorization value.
+         *
+         * If the routine fails it will return NULL and the caller will just
+         * issue a FLB_RETRY.
+         */
+        return oauth2_cache_to_token();
+    }
+    else if (ret != 0) {
         flb_plg_error(ctx->ins, "error locking mutex");
         return NULL;
     }
@@ -275,7 +354,11 @@ static flb_sds_t get_google_token(struct flb_stackdriver *ctx)
 
     /* Copy string to prevent race conditions (get_oauth2 can free the string) */
     if (ret == 0) {
-        output = flb_sds_create(ctx->o->access_token);
+        /* Update pthread keys cached values */
+        oauth2_cache_set(ctx->o->token_type, ctx->o->access_token);
+
+        /* Compose outgoing buffer using cached values */
+        output = oauth2_cache_to_token();
     }
 
     if (pthread_mutex_unlock(&ctx->token_mutex)){
@@ -539,11 +622,11 @@ static int extract_local_resource_id(const void *data, size_t bytes,
 }
 
 /*
- *    process_local_resource_id():
+ *    set_monitored_resource_labels():
  *  - use the extracted local_resource_id to assign the label keys for different
  *    resource types that are specified in the configuration of stackdriver_out plugin
  */
-static int process_local_resource_id(struct flb_stackdriver *ctx, char *type)
+static int set_monitored_resource_labels(struct flb_stackdriver *ctx, char *type)
 {
     int ret = -1;
     int first = FLB_TRUE;
@@ -551,7 +634,7 @@ static int process_local_resource_id(struct flb_stackdriver *ctx, char *type)
     int len_k8s_container;
     int len_k8s_node;
     int len_k8s_pod;
-    int prefix_len;
+    size_t prefix_len = 0;
     struct local_resource_id_list *ptr;
     struct mk_list *list = NULL;
     struct mk_list *head;
@@ -727,6 +810,25 @@ static int process_local_resource_id(struct flb_stackdriver *ctx, char *type)
     return -1;
 }
 
+static int process_local_resource_id(struct flb_stackdriver *ctx,
+                                     const char *tag, int tag_len, char *type)
+{
+    int ret;
+
+    // parsing local_resource_id from tag takes higher priority
+    if (is_tag_match_regex(ctx, tag, tag_len) > 0) {
+        ret = extract_resource_labels_from_regex(ctx, tag, tag_len, FLB_TRUE);
+    }
+    else if (is_local_resource_id_match_regex(ctx) > 0) {
+        ret = extract_resource_labels_from_regex(ctx, tag, tag_len, FLB_FALSE);
+    }
+    else {
+        ret = set_monitored_resource_labels(ctx, type);
+    }
+
+    return ret;
+}
+
 /*
  * parse_labels
  * - Iterate throught the original payload (obj) and find out the entry that matches
@@ -784,58 +886,112 @@ static void cb_results(const char *name, const char *value,
         }
         ctx->container_name = flb_sds_create_len(value, vlen);
     }
+    else if (strcmp(name, "node_name") == 0) {
+        if (ctx->node_name != NULL) {
+            flb_sds_destroy(ctx->node_name);
+        }
+        ctx->node_name = flb_sds_create_len(value, vlen);
+    }
 
     return;
 }
 
+int flb_stackdriver_regex_init(struct flb_stackdriver *ctx)
+{
+    /* If a custom regex is not set, use the defaults */
+    if (!ctx->custom_k8s_regex) {
+        ctx->custom_k8s_regex = flb_sds_create(DEFAULT_TAG_REGEX);
+        if (!ctx->custom_k8s_regex) {
+            return -1;
+        }
+    }
+
+    ctx->regex = flb_regex_create(ctx->custom_k8s_regex);
+    if (!ctx->regex) {
+        return -1;
+    }
+
+    return 0;
+}
+
 int is_tag_match_regex(struct flb_stackdriver *ctx, const char *tag, int tag_len)
 {
     int ret;
     int tag_prefix_len;
     int len_to_be_matched;
     const char *tag_str_to_be_matcheds;
-    struct flb_regex *regex;
 
     tag_prefix_len = flb_sds_len(ctx->tag_prefix);
     tag_str_to_be_matcheds = tag + tag_prefix_len;
     len_to_be_matched = tag_len - tag_prefix_len;
 
-    regex = flb_regex_create(DEFAULT_TAG_REGEX);
-    ret = flb_regex_match(regex,
+    ret = flb_regex_match(ctx->regex,
                           (unsigned char *) tag_str_to_be_matcheds,
                           len_to_be_matched);
-    flb_regex_destroy(regex);
 
     /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
     return ret;
 }
 
-/* extract_resource_labels_from_regex(3) will only be called if the
- * tag matches the regex rule
- */
-int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
-                                       const char *tag, int tag_len)
+int is_local_resource_id_match_regex(struct flb_stackdriver *ctx)
 {
-    int ret = 1;
-    int tag_prefix_len;
+    int ret;
+    int prefix_len;
     int len_to_be_matched;
-    const char *tag_str_to_be_matcheds;
-    struct flb_regex *regex;
-    struct flb_regex_search result;
+    const char *str_to_be_matcheds;
 
-    tag_prefix_len = flb_sds_len(ctx->tag_prefix);
-    tag_str_to_be_matcheds = tag + tag_prefix_len;
-    len_to_be_matched = tag_len - tag_prefix_len;
-
-    regex = flb_regex_create(DEFAULT_TAG_REGEX);
-    ret = flb_regex_do(regex, tag_str_to_be_matcheds, len_to_be_matched, &result);
-    if (ret <= 0) {
-        flb_plg_warn(ctx->ins, "invalid pattern for given tag %s", tag);
+    if (!ctx->local_resource_id) {
+        flb_plg_warn(ctx->ins, "local_resource_id not found in the payload");
         return -1;
     }
 
-    flb_regex_parse(regex, &result, cb_results, ctx);
-    flb_regex_destroy(regex);
+    prefix_len = flb_sds_len(ctx->tag_prefix);
+    str_to_be_matcheds = ctx->local_resource_id + prefix_len;
+    len_to_be_matched = flb_sds_len(ctx->local_resource_id) - prefix_len;
+
+    ret = flb_regex_match(ctx->regex,
+                          (unsigned char *) str_to_be_matcheds,
+                          len_to_be_matched);
+
+    /* 1 -> match;  0 -> doesn't match;  < 0 -> error */
+    return ret;
+}
+
+/*
+ * extract_resource_labels_from_regex(4) will only be called if the
+ * tag or local_resource_id field matches the regex rule
+ */
+int extract_resource_labels_from_regex(struct flb_stackdriver *ctx,
+                                       const char *tag, int tag_len,
+                                       int from_tag)
+{
+    int ret = 1;
+    int prefix_len;
+    int len_to_be_matched;
+    int local_resource_id_len;
+    const char *str_to_be_matcheds;
+    struct flb_regex_search result;
+
+    prefix_len = flb_sds_len(ctx->tag_prefix);
+    if (from_tag == FLB_TRUE) {
+        local_resource_id_len = tag_len;
+        str_to_be_matcheds = tag + prefix_len;
+    }
+    else {
+        // this will be called only if the payload contains local_resource_id
+        local_resource_id_len = flb_sds_len(ctx->local_resource_id);
+        str_to_be_matcheds = ctx->local_resource_id + prefix_len;
+    }
+
+    len_to_be_matched = local_resource_id_len - prefix_len;
+    ret = flb_regex_do(ctx->regex, str_to_be_matcheds, len_to_be_matched, &result);
+    if (ret <= 0) {
+        flb_plg_warn(ctx->ins, "invalid pattern for given value %s when"
+                     " extracting resource labels", str_to_be_matcheds);
+        return -1;
+    }
+
+    flb_regex_parse(ctx->regex, &result, cb_results, ctx);
 
     return ret;
 }
@@ -855,6 +1011,12 @@ static int cb_stackdriver_init(struct flb_output_instance *ins,
         return -1;
     }
 
+    /* Load config map */
+    ret = flb_output_config_map_set(ins, (void *) ctx);
+    if (ret == -1) {
+        return -1;
+    }
+
     /* Set context */
     flb_output_set_context(ins, ctx);
 
@@ -863,8 +1025,11 @@ static int cb_stackdriver_init(struct flb_output_instance *ins,
         io_flags |= FLB_IO_IPV6;
     }
 
+    /* Initialize oauth2 cache pthread keys */
+    oauth2_cache_init();
+
     /* Create mutex for acquiring oauth tokens (they are shared across flush coroutines) */
-    pthread_mutex_init ( &ctx->token_mutex, NULL);
+    pthread_mutex_init(&ctx->token_mutex, NULL);
 
     /* Create Upstream context for Stackdriver Logging (no oauth2 service) */
     ctx->u = flb_upstream_create_url(config, FLB_STD_WRITE_URL,
@@ -897,7 +1062,8 @@ static int cb_stackdriver_init(struct flb_output_instance *ins,
         token = get_google_token(ctx);
         if (!token) {
             flb_plg_warn(ctx->ins, "token retrieval failed");
-        } else {
+        }
+        else {
             flb_sds_destroy(token);
         }
     }
@@ -931,6 +1097,12 @@ static int cb_stackdriver_init(struct flb_output_instance *ins,
         ctx->export_to_project_id = flb_sds_create(ctx->project_id);
     }
 
+    ret = flb_stackdriver_regex_init(ctx);
+    if (ret == -1) {
+        flb_plg_error(ctx->ins, "failed to init stackdriver custom regex");
+        return -1;
+    }
+
     return 0;
 }
 
@@ -1046,37 +1218,6 @@ static int get_severity_level(severity_t * s, const msgpack_object * o,
     return -1;
 }
 
-
-static int get_stream(msgpack_object_map map)
-{
-    int i;
-    int len_stdout;
-    int val_size;
-    msgpack_object k;
-    msgpack_object v;
-
-    /* len(stdout) == len(stderr) */
-    len_stdout = sizeof(STDOUT) - 1;
-    for (i = 0; i < map.size; i++) {
-        k = map.ptr[i].key;
-        v = map.ptr[i].val;
-        if (k.type == MSGPACK_OBJECT_STR &&
-            strncmp(k.via.str.ptr, "stream", k.via.str.size) == 0) {
-            val_size = v.via.str.size;
-            if (val_size == len_stdout) {
-                if (strncmp(v.via.str.ptr, STDOUT, val_size) == 0) {
-                    return STREAM_STDOUT;
-                }
-                else if (strncmp(v.via.str.ptr, STDERR, val_size) == 0) {
-                    return STREAM_STDERR;
-                }
-            }
-        }
-    }
-
-    return STREAM_UNKNOWN;
-}
-
 static insert_id_status validate_insert_id(msgpack_object * insert_id_value,
                                            const msgpack_object * obj)
 {
@@ -1304,7 +1445,6 @@ static int stackdriver_format(struct flb_config *config,
     int array_size = 0;
     /* The default value is 3: timestamp, jsonPayload, logName. */
     int entry_size = 3;
-    int stream;
     size_t s;
     size_t off = 0;
     char path[PATH_MAX];
@@ -1332,6 +1472,8 @@ static int stackdriver_format(struct flb_config *config,
     /* Parameters for log name */
     int log_name_extracted = FLB_FALSE;
     flb_sds_t log_name = NULL;
+    flb_sds_t stream = NULL;
+    flb_sds_t stream_key;
 
     /* Parameters for insertId */
     msgpack_object insert_id_obj;
@@ -1515,13 +1657,7 @@ static int stackdriver_format(struct flb_config *config,
              *    k8s_container.<namespace_name>.<pod_name>.<container_name>
              */
 
-            if (is_tag_match_regex(ctx, tag, tag_len) > 0) {
-                ret = extract_resource_labels_from_regex(ctx, tag, tag_len);
-            }
-            else {
-                ret = process_local_resource_id(ctx, K8S_CONTAINER);
-            }
-
+            ret = process_local_resource_id(ctx, tag, tag_len, K8S_CONTAINER);
             if (ret == -1) {
                 flb_plg_error(ctx->ins, "fail to extract resource labels "
                               "for k8s_container resource type");
@@ -1574,8 +1710,8 @@ static int stackdriver_format(struct flb_config *config,
              *      k8s_node.<node_name>
              */
 
-            ret = process_local_resource_id(ctx, K8S_NODE);
-            if (ret != 0) {
+            ret = process_local_resource_id(ctx, tag, tag_len, K8S_NODE);
+            if (ret == -1) {
                 flb_plg_error(ctx->ins, "fail to process local_resource_id from "
                               "log entry for k8s_node");
                 msgpack_sbuffer_destroy(&mp_sbuf);
@@ -1616,7 +1752,7 @@ static int stackdriver_format(struct flb_config *config,
              *      k8s_pod.<namespace_name>.<pod_name>
              */
 
-            ret = process_local_resource_id(ctx, K8S_POD);
+            ret = process_local_resource_id(ctx, tag, tag_len, K8S_POD);
             if (ret != 0) {
                 flb_plg_error(ctx->ins, "fail to process local_resource_id from "
                               "log entry for k8s_pod");
@@ -1862,12 +1998,13 @@ static int stackdriver_format(struct flb_config *config,
 
         /* avoid modifying the original tag */
         newtag = tag;
-        if (ctx->is_k8s_resource_type) {
-            stream = get_stream(result.data.via.array.ptr[1].via.map);
-            if (stream == STREAM_STDOUT) {
+        stream_key = flb_sds_create("stream");
+        if (ctx->is_k8s_resource_type
+            && get_string(&stream, obj, stream_key) == 0) {
+            if (flb_sds_cmp(stream, STDOUT, flb_sds_len(stream)) == 0) {
                 newtag = "stdout";
             }
-            else if (stream == STREAM_STDERR) {
+            else if (flb_sds_cmp(stream, STDERR, flb_sds_len(stream)) == 0) {
                 newtag = "stderr";
             }
         }
@@ -1891,6 +2028,8 @@ static int stackdriver_format(struct flb_config *config,
         msgpack_pack_str_body(&mp_pck, "logName", 7);
         msgpack_pack_str(&mp_pck, len);
         msgpack_pack_str_body(&mp_pck, path, len);
+        flb_sds_destroy(stream_key);
+        flb_sds_destroy(stream);
 
         /* timestamp */
         msgpack_pack_str(&mp_pck, 9);
@@ -1933,17 +2072,6 @@ static int stackdriver_format(struct flb_config *config,
     return 0;
 }
 
-static void set_authorization_header(struct flb_http_client *c,
-                                     char *token)
-{
-    int len;
-    char header[512];
-
-    len = snprintf(header, sizeof(header) - 1,
-                   "Bearer %s", token);
-    flb_http_add_header(c, "Authorization", 13, header, len);
-}
-
 static void cb_stackdriver_flush(const void *data, size_t bytes,
                                  const char *tag, int tag_len,
                                  struct flb_input_instance *i_ins,
@@ -2009,9 +2137,7 @@ static void cb_stackdriver_flush(const void *data, size_t bytes,
     }
 
     flb_http_add_header(c, "Content-Type", 12, "application/json", 16);
-
-    /* Compose and append Authorization header */
-    set_authorization_header(c, token);
+    flb_http_add_header(c, "Authorization", 13, token, flb_sds_len(token));
 
     /* Send HTTP request */
     ret = flb_http_do(c, &b_sent);
@@ -2027,6 +2153,9 @@ static void cb_stackdriver_flush(const void *data, size_t bytes,
         if (c->resp.status == 200) {
             ret_code = FLB_OK;
         }
+        else if (c->resp.status >= 400 && c->resp.status < 500) {
+            ret_code = FLB_ERROR;
+        }
         else {
             if (c->resp.payload_size > 0) {
                 /* we got an error */
